%{
/* $Id: uncomment.l,v 1.2 1995/06/14 15:02:41 sgp13 Exp $

   -----------------------------------------------------------------

   $Log: uncomment.l,v $
 * Revision 1.2  1995/06/14  15:02:41  sgp13
 * Changed to process read-only files, but to preserve their status on exit.
 *
 * Revision 1.1  1995/06/14  10:24:49  sgp13
 * Initial revision
 *
*/

/* this is a Lex program to uncomment all Diag.Message calls from the */
/* file on the standard input */

/* assumptions:

   that the sequence ); may be taken to terminate the Diag.Message call

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#define FALSE 0
#define TRUE  1

/* flag whether in the middle of processing a Diag.Message command */
int  uncommenting = FALSE;

#define ERROR 1

%}

bra           \(
ket           \)
semicolon     \;
star          \*
opencomment   {bra}{star}
closecomment  {star}{ket}
procedure     Diag\.Message
endstatement  {ket}{semicolon}

%%
{opencomment}{procedure} { /* commented Diag.Message */
                           /* uncomment and set flag */
                           if (uncommenting == FALSE) {
                             fputs("Diag.Message", yyout);
                             uncommenting = TRUE;
                           }
                           else {
                             fprintf(stderr, "error\n");
                             return ERROR;
                           }                             
                         }

{procedure}              { /* not commented Diag.Message */
                           /* echo */
                           fputs("Diag.Message", yyout);
                         }

{endstatement}{closecomment}  { /* end of already commented command */
                                /* uncomment */
                                fputs(");", yyout);
                                if (uncommenting == TRUE)
                                  uncommenting = FALSE;
                                else
                                  fputs(");*)", yyout);
                              }

{endstatement}           { /* end of statement */
                           fputs(");", yyout);
                         }
%%
/* main program for the commenter/uncommenter utility

*/

#define MAX_FILENAME    200
#define TEMPFILE_PATH   "/home/sgp13"
#define TEMPFILE_PREFIX "com"

void main(int argc, char **argv)
{
  FILE *filelist;
  char *tempfilename;
  char inputfilename[MAX_FILENAME];
  char *nlpos;              /* pointer to '\n' in string read from filelist */
  int result;
  struct stat statbuf;

  /* check parameters */
  if (argc != 2) {
    printf("comment: comment out Diag.Message() statements\n\n");
    printf("usage: comment <file containing list of filenames>\n\n");
    exit(1);
  }

  /* open file which lists the files to process */
  if ((filelist = fopen(argv[1], "r")) == (FILE*)NULL) {
    printf("comment: could not open %s\n", argv[1]);
    exit(2);
  }

  /* obtain temporary filename for lexer to use */
  tempfilename = tempnam(TEMPFILE_PATH, TEMPFILE_PREFIX);

  /* for each file in the list file */
  while (!feof(filelist)) {
    /* read filename */
    if (fgets(inputfilename, MAX_FILENAME, filelist) == NULL)
      break;

    /* discard newline character */
    nlpos = strchr(inputfilename, '\n');
    if (nlpos != NULL)
      *nlpos = '\0';

    /* open input and temporary output */
    if ((yyin = fopen(inputfilename, "r")) == NULL) {
      printf("comment: could not open input file %s: skipping\n",
             inputfilename);
      continue;
    }

    if ((yyout = fopen(tempfilename, "w")) == NULL) {
      printf("comment: could not open file for output: fatal error\n");
      remove(tempfilename);
      fclose(filelist);
      fclose(yyin);
      exit(3);
    }

    /* call lexer to process */
    result = yylex();

    /* close files */
    fclose(yyin);
    fclose(yyout);

    /* if fails then say so: success is a return of zero */
    /* otherwise move temporary output onto input */
    if (result == ERROR) {
      printf("comment: processing failed on file %s: skipped\n",
             inputfilename);
    }
    else {
      /* read the file attributes */
      if (stat(inputfilename, &statbuf) == -1) {
        printf("comment: stat() failed: fatal error\n");
        exit(4);
      }

      /* make file writable */
      if (chmod(inputfilename, statbuf.st_mode | S_IWUSR) == -1) {
        printf("comment: chmod() failed: fatal error\n");
        exit(5);
      }

      if (rename(tempfilename, inputfilename) != 0) {
        printf("comment: rename() failed on file %s\n", inputfilename);
        printf("         source file probably readonly: file skipped\n");
        perror(NULL);
      }

      /* restore file attributes */
      if (chmod(inputfilename, statbuf.st_mode) == -1) {
        printf("comment: chmod() failed: fatal error\n");
        exit(5);
      }
      
    }

    remove(tempfilename);

  }

  fclose(filelist);
  free(tempfilename);
  exit(0);

}


