(* ----------------------------------------------------------------------
   $Id: BINtoCore.m3,v 1.2 1995/01/11 10:33:04 sgp13 Exp sgp13 $

   BIN format to core dump format converter. This program reads a file in
   BIN loader form and inserts the information into a memory object which
   is then saved to the specified file.
   ----------------------------------------------------------------------

   $Log: BINtoCore.m3,v $
# Revision 1.2  1995/01/11  10:33:04  sgp13
# Update for version 1.3 of Memory
#
# Revision 1.1  1995/01/10  13:01:39  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE BINtoCore EXPORTS Main;

IMPORT IO, Wr, FileWr, Rd, FileRd, Params;
IMPORT Memory;

VAR
  memory := NEW(Memory.T).init(NIL); (* an initially blank memory object *)
  field : Memory.FieldNumber := 0;
  address : Memory.Address;
  input : Rd.T;            (* input file stream *)
  output : Wr.T;           (* output file stream *)
  ch : INTEGER;            (* store for character from file *)
  addressSet := FALSE;     (* flag whether the address has been set *)
  value : Memory.Value;

EXCEPTION AddressNotSet;

PROCEDURE Decode(ch : INTEGER; input : Rd.T) : INTEGER =
  BEGIN
    RETURN (ch MOD 8_100) * 8_100 + (ORD(Rd.GetChar(input)) MOD 8_100);
  END Decode;

BEGIN
  IF Params.Count # 3 THEN
    IO.Put("usage: BINtoCore <BIN file> <memory dump file>\n");
  ELSE
    (* open input and output files *)
    input := FileRd.Open(Params.Get(1));
    output := FileWr.Open(Params.Get(2));

    TRY
      (* process each character in the file *)
      WHILE NOT Rd.EOF(input) DO
        (* read character *)
        ch := ORD(Rd.GetChar(input));

        (* classify character depending on value of top two bits *)
        CASE ch DIV 8_100 OF
        | 0 => (* ordinary character - first of two byte pair specifying a *)
               (* value for the memory location *)
               (* an error will occur if an attempt is made to write beyond *)
               (* the end of a field *)
               value := Decode(ch, input);
               IF addressSet THEN
                 memory.mem[field]^[address] := value;
                 IF address = LAST(Memory.Address) THEN
                   address := 0;
                 ELSE
                   INC(address);
                 END;
               ELSE
                 RAISE AddressNotSet;
               END;
             
        | 1 => (* address - first of two byte pair specifying the next *)
               (* address to write to *)
               address := Decode(ch, input);
               addressSet := TRUE;

        | 2 => (* blank *)
        | 3 => (* field specifier *)
               field := (ch DIV 8_10) MOD 8_10;
        END;

      END;

      (* dump the resulting memory image *)
      memory.dump(output);

    EXCEPT
    | AddressNotSet => IO.Put("Value specified before setting address\n");
                       IO.Put("Invalid format for BIN file\n");
    END;

    Rd.Close(input);
    Wr.Close(output);

  END;

END BINtoCore.
