(* ----------------------------------------------------------------------
   $Id: GUISwitchRegister.m3,v 1.2 1995/06/23 07:52:04 sgp13 Exp $

   The procedures to manipulate the GUI switch register

   The names of the procedures indicate the action which they perform:
   ie BitZeroOn, sets bit zero of the SR to '1'
   ----------------------------------------------------------------------

   $Log: GUISwitchRegister.m3,v $
# Revision 1.2  1995/06/23  07:52:04  sgp13
# Registers are now accessed directly
#
# Revision 1.1  1995/03/23  09:47:09  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE GUISwitchRegister;

IMPORT FormsVBT, VBT, Word;
IMPORT Bits, CPU, Registers;

<* FATAL Registers.OutOfRange *>

VAR
  (* pointer to the CPU object to be able to change SR *)
  cpu : CPU.T;


(* ----------------------------------------------------------------------
   Procedure : ModuleInit

   Set up pointer to the CPU object
   ---------------------------------------------------------------------- *)
PROCEDURE ModuleInit(object : CPU.T) =
  BEGIN
    cpu := object;
  END ModuleInit;


(* ----------------------------------------------------------------------
   Procedure : BitZeroOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitZeroOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitZero);
    END;
  END BitZeroOn;


(* ----------------------------------------------------------------------
   Procedure : BitZeroOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitZeroOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitZero);
    END;
  END BitZeroOff;


(* ----------------------------------------------------------------------
   Procedure : BitOneOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitOneOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitOne);
    END;
  END BitOneOn;


(* ----------------------------------------------------------------------
   Procedure : BitOneOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitOneOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitOne);
    END;
  END BitOneOff;


(* ----------------------------------------------------------------------
   Procedure : BitTwoOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitTwoOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitTwo);
    END;
  END BitTwoOn;


(* ----------------------------------------------------------------------
   Procedure : BitTwoOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitTwoOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitTwo);
    END;
  END BitTwoOff;


(* ----------------------------------------------------------------------
   Procedure : BitThreeOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitThreeOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitThree);
    END;
  END BitThreeOn;


(* ----------------------------------------------------------------------
   Procedure : BitThreeOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitThreeOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitThree);
    END;
  END BitThreeOff;


(* ----------------------------------------------------------------------
   Procedure : BitFourOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitFourOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitFour);
    END;
  END BitFourOn;


(* ----------------------------------------------------------------------
   Procedure : BitFourOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitFourOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitFour);
    END;
  END BitFourOff;


(* ----------------------------------------------------------------------
   Procedure : BitFiveOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitFiveOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitFive);
    END;
  END BitFiveOn;


(* ----------------------------------------------------------------------
   Procedure : BitFiveOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitFiveOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitFive);
    END;
  END BitFiveOff;


(* ----------------------------------------------------------------------
   Procedure : BitSixOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitSixOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitSix);
    END;
  END BitSixOn;


(* ----------------------------------------------------------------------
   Procedure : BitSixOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitSixOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitSix);
    END;
  END BitSixOff;


(* ----------------------------------------------------------------------
   Procedure : BitSevenOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitSevenOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitSeven);
    END;
  END BitSevenOn;


(* ----------------------------------------------------------------------
   Procedure : BitSevenOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitSevenOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitSeven);
    END;
  END BitSevenOff;


(* ----------------------------------------------------------------------
   Procedure : BitEightOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitEightOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitEight);
    END;
  END BitEightOn;


(* ----------------------------------------------------------------------
   Procedure : BitEightOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitEightOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitEight);
    END;
  END BitEightOff;


(* ----------------------------------------------------------------------
   Procedure : BitNineOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitNineOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitNine);
    END;
  END BitNineOn;


(* ----------------------------------------------------------------------
   Procedure : BitNineOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitNineOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitNine);
    END;
  END BitNineOff;


(* ----------------------------------------------------------------------
   Procedure : BitTenOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitTenOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitTen);
    END;
  END BitTenOn;


(* ----------------------------------------------------------------------
   Procedure : BitTenOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitTenOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitTen);
    END;
  END BitTenOff;


(* ----------------------------------------------------------------------
   Procedure : BitElevenOn

   Switch on the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitElevenOn(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.Or(SR, Bits.setBitEleven);
    END;
  END BitElevenOn;


(* ----------------------------------------------------------------------
   Procedure : BitElevenOff

   Switch off the specified bit of the switch register
   ---------------------------------------------------------------------- *)
PROCEDURE BitElevenOff(<* UNUSED *> form : FormsVBT.T := NIL;
                 <* UNUSED *> event : TEXT := NIL;
                 <* UNUSED *> data : REFANY := NIL;
                 <* UNUSED *> time : VBT.TimeStamp := 0) =
  BEGIN
    WITH SR = cpu.SR DO
      SR := Word.And(SR, Bits.clearBitEleven);
    END;
  END BitElevenOff;

BEGIN
END GUISwitchRegister.

