(* ----------------------------------------------------------------------
   $Id: SimpleInstruction.m3,v 1.10 1995/06/24 21:58:08 sgp13 Exp $

   Implement the 'simple' instructions - ie those which contain a memory
   reference.
   ----------------------------------------------------------------------

   $Log: SimpleInstruction.m3,v $
# Revision 1.10  1995/06/24  21:58:08  sgp13
# Memory split into fields so that current field pointers may be used to speed
# access
#
# Revision 1.9  1995/06/23  14:13:44  sgp13
# CPU.Quit not raised
#
# Revision 1.8  1995/06/23  07:56:41  sgp13
# Registers are now accessed directly
#
# Revision 1.7  1995/06/22  14:06:10  sgp13
# Memory made direct access
#
# Revision 1.6  1995/06/20  10:17:42  sgp13
# Changes due to IntBus version 1.2
#
# Revision 1.5  1995/03/22  13:21:46  sgp13
# TAD corrected
#
# Revision 1.4  1995/03/15  14:55:20  sgp13
# Change in interrupt method (to use IntBus.T) implied change to JMP and JMS
#
# Revision 1.3  1995/03/14  12:30:26  sgp13
# Corrected field in which JMS return address is written.
#
# Revision 1.2  1995/02/24  22:31:09  sgp13
# CPUdata renamed CPU
#
# Revision 1.1  1995/01/23  13:12:17  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE SimpleInstruction;

IMPORT Word;
IMPORT Diag, Memory, Registers;
IMPORT CPU;

<* FATAL Registers.OutOfRange *>

(* ----------------------------------------------------------------------
   Procedure : DoAND

   Procedure to execute AND instruction
   ---------------------------------------------------------------------- *)
PROCEDURE DoAND(self : CPU.T; opaddress : Memory.Address;
                opfield : REF Memory.Field) =
  BEGIN
    WITH AC = self.AC DO
      AC := Word.And(AC, opfield[opaddress]);
      (*Diag.Message(Diag.T.Instruction, "AND");*)
    END;
  END DoAND;



(* ----------------------------------------------------------------------
   Procedure : DoTAD

   Procedure to execute TAD instruction
   The method used is as described in article 1159 from the newsgroup
   alt.sys.pdp8 a copy of which is in the documentation directory.
   Perform addition and complement link if overflow occurs
   ---------------------------------------------------------------------- *)
PROCEDURE DoTAD(self : CPU.T; opaddress : Memory.Address;
	        opfield : REF Memory.Field) =
  VAR
    result : INTEGER;
  BEGIN
    WITH AC = self.AC, L = self.L DO

      IF L THEN
        result := 8_10000;
      ELSE
        result := 0;
      END;

      result := result + AC + opfield[opaddress];

      IF Word.And(result, Memory.byteSize) # 0 THEN
        L := TRUE;
        AC := result MOD Memory.byteSize;
      ELSE
        L := FALSE;
        AC := result MOD Memory.byteSize;
      END;
    END;
    (*Diag.Message(Diag.T.Instruction, "TAD");*)
  END DoTAD;



(* ----------------------------------------------------------------------
   Procedure : DoISZ

   Procedure to execute ISZ instruction
   Increment location and skip next instruction if the location is
   incremented to zero
   ---------------------------------------------------------------------- *)
PROCEDURE DoISZ(self : CPU.T; opaddress : Memory.Address;
                opfield : REF Memory.Field) =
  BEGIN
    WITH value = opfield[opaddress] DO
      IF value = LAST(Memory.Value) THEN
        value := 0;
        (* skip next instruction *)
        IF self.PC = LAST(CPU.Register_12) THEN
          self.PC := 0
        ELSE
          INC(self.PC);
        END;
        (*Diag.Message(Diag.T.Instruction, "ISZ(skip)");*)
      ELSE
        (*Diag.Message(Diag.T.Instruction, "ISZ(noskip)");*)
        INC(value);
      END;
    END;
  END DoISZ;



(* ----------------------------------------------------------------------
   Procedure : DoDCA

   Procedure to execute DCA instruction
   Deposit contents of accumulator to memory and clear accumulator
   ---------------------------------------------------------------------- *)
PROCEDURE DoDCA(self : CPU.T; opaddress : Memory.Address;
                opfield : REF Memory.Field) =
  BEGIN
    WITH AC = self.AC DO
      opfield[opaddress] := AC;
      AC := 0;
    END;
    (*Diag.Message(Diag.T.Instruction, "DCA");*)
  END DoDCA;



(* ----------------------------------------------------------------------
   Procedure : DoJMS

   Procedure to execute JMS instruction
   ---------------------------------------------------------------------- *)
PROCEDURE DoJMS(self : CPU.T; opaddress : Memory.Address;
                <* UNUSED *> opfield: REF Memory.Field) =
  BEGIN
    (*Diag.Message(Diag.T.Instruction, "JMS");*)
    (* if there is a transfer from instruction save field pending then *)
    (* perform it and uninhibit interrupts *)
    WITH interrupt = self.interrupt DO
      IF NOT interrupt.uninhibited THEN
        interrupt.uninhibited := TRUE;
        self.EM_IFnum := self.EM_IB;
        self.EM_IFptr := self.memory.mem[self.EM_IFnum];
      END;
    END; 
    (* store the current PC as a return address *)
    (* set the PC to the start of the subroutine proper, i.e. the following *)
    (* location *)
    WITH PC = self.PC DO
      self.EM_IFptr[opaddress] := PC;
      PC := opaddress;
      IF PC = LAST(CPU.Register_12) THEN
        PC := 0;
      ELSE
        INC(PC);
      END;
    END;
  END DoJMS;



(* ----------------------------------------------------------------------
   Procedure : DoJMP

   Procedure to execute JMP instruction
   ---------------------------------------------------------------------- *)
PROCEDURE DoJMP(self : CPU.T; opaddress : Memory.Address;
                <* UNUSED *> opfield : REF Memory.Field) =
  BEGIN
    (*Diag.Message(Diag.T.Instruction, "JMP");*)
    (* if there is a transfer from instruction save field pending then *)
    (* perform it and uninhibit interrupts *)
    WITH interrupt = self.interrupt DO
      IF NOT interrupt.uninhibited THEN
        interrupt.uninhibited := TRUE;
        self.EM_IFnum := self.EM_IB;
        self.EM_IFptr := self.memory.mem[self.EM_IFnum];
      END;
    END; 
    (* jump to new address *)
    self.PC := opaddress;
  END DoJMP;


BEGIN
END SimpleInstruction.
