(* ----------------------------------------------------------------------
   $Id: MemDisp.m3,v 1.3 1995/03/07 20:50:22 sgp13 Exp sgp13 $

   Program to display selected contents of a memory dump file.
   The contents to be displayed are selected by a field and range of
   addresses specified on the command line.

   NB Numbers are specified in OCTAL
   ----------------------------------------------------------------------

   $Log: MemDisp.m3,v $
# Revision 1.3  1995/03/07  20:50:22  sgp13
# Changed to print the ASCII characters associated with memory values if
# appropriate.
#
# Also now reports its identity and version number.
#
# Revision 1.2  1995/01/11  10:30:28  sgp13
# Update for use with version 1.3 of Memory module
#
# Revision 1.1  1995/01/10  12:32:44  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE MemDisp EXPORTS Main;

IMPORT ASCII, Params, Scan, IO, Rd, FileRd, Fmt, Lex, OSError, Text;
IMPORT Memory;

CONST
  lineWidth = 8;  (* specifies the number of values to display per line *)

VAR
  memory : Memory.T;
  numberOnLine := 0;  (* the number of values so far on the current line *)

EXCEPTION StartAfterEnd; AddrsOutOfRange; FieldOutOfRange;

BEGIN
  IO.Put("\nMemDisp: PDP8/e core memory dump: version 1.4\n\n");

  IF Params.Count # 5 THEN
    IO.Put("usage: MemDisp <file> <field> <start> <end>\n");
    IO.Put("NB numbers to be specified in OCTAL\n");
  ELSE
    TRY
      WITH file = FileRd.Open(Params.Get(1)),
           field = Scan.Int(Params.Get(2)),
           start = Scan.Int(Params.Get(3), 8),
           end = Scan.Int(Params.Get(4), 8) DO

        (* check parameters *)
        IF start > end THEN
          RAISE StartAfterEnd;
        END;
        IF start < 0 OR start >= Memory.fieldSize
            OR end < 0 OR end >= Memory.fieldSize THEN
          RAISE AddrsOutOfRange;
        END;
        IF field < 0 OR field >= Memory.maxFields THEN
          RAISE FieldOutOfRange;
        END;

        (* load memory image *)
        memory := NEW(Memory.T).init(file);

        (* print header *)
        IO.Put("Dump from " & Params.Get(1) & " field "
               & Fmt.Int(field) & "\n\n");

        (* output *)
        FOR lineStart := start TO end BY lineWidth DO

          (* print address at start of line *)
          IF numberOnLine = 0 THEN
            IO.Put(Fmt.Pad(Fmt.Int(lineStart, 8), 4, '0') & " : ");
          END;

          FOR address := lineStart TO lineStart + lineWidth - 1 DO
            IF address < 8_10000 THEN
              IO.Put(Fmt.Pad(Fmt.Int(memory.mem[field]^[address],8),
                             4,'0') & " ");
            END;
          END;

          IO.Put(" : ");

          FOR address := lineStart TO lineStart + lineWidth - 1 DO
            IF address < 8_10000 THEN
              WITH value = memory.mem[field]^[address] DO
                IF value < 256 AND NOT (VAL(value, CHAR) IN ASCII.Controls)THEN
                  IO.Put(Text.FromChar(VAL(value, CHAR)));
                ELSE
                  IO.Put(".");
                END;
              END;
            END;
          END;

          IO.Put("\n");

        END;

        IO.Put("\n");

        Rd.Close(file);
      END;
    EXCEPT
    | StartAfterEnd => IO.Put("MemDisp: Invalid range of addresses " &
                              "requested\n");
    | AddrsOutOfRange => IO.Put("MemDisp: Invalid address(es) specified\n");
    | FieldOutOfRange => IO.Put("MemDisp: Invalid field number specified\n");
    | Lex.Error => IO.Put("MemDisp: Invalid parameter specified\n");
    | OSError.E => IO.Put("MemDisp: Error opening file\n");
    END;
  END;

  IO.Put("\n");

END MemDisp.





