(* ----------------------------------------------------------------------
   $Id$

   Make a memory image file from an ASCII memory dump as produced by
   MemDisp or coredump.

   The formats accepted are:

   MemDisp: 'xxxx : xxxx xxxx xxxx xxxx xxxx xxxx xxxx xxxx : xxxxxxxx'
   coredump: 'xxxxx: xxxx xxxx xxxx xxxx xxxx xxxx xxxx xxxx'

   NB the leading lines of the MemDisp output must be removed before
      submitting the file to MemMake

      the fields are taken to be contiguous, so that the output of MemDisp
      will always go into field zero. An address 1xxxx will go into field 1.

   ----------------------------------------------------------------------

   $Log$

   ---------------------------------------------------------------------- *)
MODULE MemMake EXPORTS Main;

IMPORT IO, Rd, Wr;
IMPORT Memory;
IMPORT Utils;

CONST
  version = "1.0";

VAR
  (* create initially blank memory object *)
  memory := NEW(Memory.T).init(NIL);

  (* file streams *)
  source : Rd.T;
  destination : Wr.T;

  (* current line of file *)
  line : TEXT;

BEGIN
  (* report identity *)
  IO.Put("MemMake: version " & version & "\n\n");
  IO.Put("Create memory image from ASCII dump file\n\n");

  (* check syntax of command line and extract names *)
  (* open files for input and output *)
  (* NB REFERENCE parameters produce desired side effects *)
  Utils.AnalyseCommand(source, destination);

  (* while not EOF *)
  TRY
    WHILE NOT Rd.EOF(source) DO
      (* read line *)
      line := Rd.GetLine(source);

      (* decode line and insert in memory object *)
      Utils.ProcessLine(line, memory);
    END;
  EXCEPT
    Rd.EndOfFile =>
  END;

  (* dump memory image *)
  memory.dump(destination);

  (* close files *)
  Rd.Close(source);
  Wr.Close(destination);

END MemMake.
