(* ----------------------------------------------------------------------
   $Id$

   Utility procedures for MemMake program
   ----------------------------------------------------------------------

   $Log$

   ---------------------------------------------------------------------- *)
MODULE Utils;

IMPORT ASCII, FileRd, FileWr, IO, OSError, Params, Rd, Scan, Text, Wr;
IMPORT Memory;

(* ----------------------------------------------------------------------
   Procedure : AnalyseCommand

   Analyse the command line and open the files, returning the file streams
   to the caller.
   ---------------------------------------------------------------------- *)
PROCEDURE AnalyseCommand(VAR source : Rd.T; VAR destination : Wr.T) =
  VAR
    sourcename : TEXT;
    destname : TEXT;
  BEGIN
    IF Params.Count # 3 THEN
      IO.Put("syntax: MemMake <source file> <destination file>\n\n");
      RAISE CommandSyntax;
    END;

    sourcename := Params.Get(1);
    destname := Params.Get(2);

    (* attempt to open source and destination files *)
    TRY
      source := FileRd.Open(sourcename);
    EXCEPT
      OSError.E => IO.Put("could not open " & sourcename & "\n");
      RAISE FilesNotOpened;
    END;

    TRY
      destination := FileWr.Open(destname);
    EXCEPT
      OSError.E => IO.Put("could not open " & destname & "\n");
      Rd.Close(source);
      RAISE FilesNotOpened;
    END;

  END AnalyseCommand;


(* ----------------------------------------------------------------------
   Procedure : ProcessLine

   Process a line from the input file
   ---------------------------------------------------------------------- *)
PROCEDURE ProcessLine(line : TEXT; memory : Memory.T) =
  VAR
    start : CARDINAL := 0; (* position in line of first value *)
    address : Memory.Address;
    field : Memory.FieldNumber;
    value : CARDINAL;
    scratch : TEXT;       (* temporary store for numbers extracted from line *)
  BEGIN
    (* read address from beginning of line *)
    IF Text.GetChar(line, 4) = ':' THEN
      scratch := Text.Sub(line, 0, 4);
    ELSE
      scratch := Text.Sub(line, 0, 5);
    END;

    value := Scan.Int(scratch, 8);

    address := value MOD Memory.fieldSize;
    field := value DIV Memory.fieldSize;

    (* skip from position five in line to first digit *)
    start := 5;

    WHILE NOT (Text.GetChar(line, start) IN ASCII.Digits) DO
      INC(start);
    END;

    (* read eight numbers: *)
    FOR offset := start TO start + 35 BY 5 DO
      (* read number and translate and store *)
      scratch := Text.Sub(line, offset, 5);
      memory.mem[field]^[address] := Scan.Int(scratch, 8);
      IF address # LAST(Memory.Address) THEN
        INC(address);
      ELSE
        address := 0;
      END;
    END;

  END ProcessLine;

BEGIN
END Utils.






