(* ----------------------------------------------------------------------
   $Id: Bus.i3,v 1.6 1995/06/23 07:44:55 sgp13 Exp $

   This interface specifies the services which the bus provides. To implement
   an IOT command, the CPU thread invokes the Bus.T.command method passing
   as parameters, the CPU object, and the instruction.

   The other methods supplied by the bus are:

   init - to initialise the bus object
   reset - to implement the CAF command (logically equivalently, this also
           implements depression of the CLEAR key)

   The objects on the bus must be visible to the outside world, so that
   auxilliary services such as mounting a file on the punch/reader/teletype
   can be easily implemented.

   It is the responsibility of the bus to trap the exceptions raised if a
   device or command is detected as being unimplemented and to produce
   appropriate diagnostics messages.

   ----------------------------------------------------------------------

   $Log: Bus.i3,v $
# Revision 1.6  1995/06/23  07:44:55  sgp13
# Update forced
#
# Revision 1.5  1995/03/15  14:43:39  sgp13
# Update forced
#
# Revision 1.4  1995/03/14  12:13:30  sgp13
# Update forced
#
# Revision 1.3  1995/03/07  23:05:51  sgp13
# Tape and Teletype implemented
#
# Revision 1.2  1995/02/24  22:14:11  sgp13
# Interrupts and Extended memory implemented
#
# Revision 1.1  1995/02/21  09:38:34  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE Bus;

IMPORT Thread;
IMPORT CPU;
IMPORT Device, ExtdMem, Interrupts, Tape, Teletype;


(* this is the bus object as described *)
TYPE T = OBJECT
  extdmem : ExtdMem.T;
  interrupts : Interrupts.T;

  disc : Device.T;
  oscilloscope : Device.T;

  teletype : Teletype.T;
  tape : Tape.T;

METHODS
  command(cpu : CPU.T; instruction : INTEGER) RAISES { Thread.Alerted }
                               := Command;
  init() : T := Init;
  reset() := Reset;
END;

PROCEDURE Command(self : T; cpu : CPU.T; instruction : INTEGER)
    RAISES { Thread.Alerted };
PROCEDURE Init(self : T) : T;
PROCEDURE Reset(self : T);

END Bus.


