(* ----------------------------------------------------------------------
   $Id: CharReader.i3,v 1.5 1995/06/19 13:58:55 sgp13 Exp $

   Interface supplying the character reader object and associated thread
   applying procedure. This object implements reading characters from an
   attached input stream for both the Teletype and Tape objects
   ----------------------------------------------------------------------

   $Log: CharReader.i3,v $
# Revision 1.5  1995/06/19  13:58:55  sgp13
# Reader is now a CharRd.T type standard interface to either a BufferRd.T or
# a StreamRd.T
#
# Revision 1.4  1995/05/03  21:29:55  sgp13
# Update forced
#
# Revision 1.3  1995/03/15  14:44:46  sgp13
# Interrupts now implemented using IntBus.T
#
# Revision 1.2  1995/03/14  12:14:53  sgp13
# Interrupts method altered to counting pending interrupts.
#
# Revision 1.1  1995/02/24  23:27:06  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE CharReader;

IMPORT Thread;
IMPORT CPU, CharRd, Flags, IntBus;

TYPE
  T = Thread.Closure OBJECT
    byte : INTEGER;         (* the value of the next byte read *)
    cpu : CPU.T;            (* the CPU object *)
    interruptFlag : BOOLEAN := FALSE;
                            (* flag whether setting the ready flag *)
                            (* causes an interrupt *)
    interruptID : IntBus.ID;(* the number of the interrupt produced by *)
                            (* the device *)
    ready : BOOLEAN := FALSE; (* indicates that 'byte' is valid *)
    readyLock : MUTEX;      (* lock on the ready flag *)
    run : Flags.Signal;     (* signal to fetch next byte *)
    reader : CharRd.T;      (* source to read from *)
    teletype : BOOLEAN;     (* flag whether teletype, and hence whether *)
                            (* to automatically set the top bit of any *)
                            (* character read *)
  METHODS
    connect(input : CharRd.T) RAISES { CharRd.AlreadyConnected } := Connect;
                                         (* connect an input to the object *)
    disconnect() : CharRd.T := Disconnect;   (* disconnect the input *)
    init(cpu : CPU.T; teletype : BOOLEAN) : T := ThreadInit;
  OVERRIDES
    apply := DoCharReader;
  END;


(* ----------------------------------------------------------------------
   Procedure : Connect

   Connect the specified file stream to the reader. Raise an exception if
   there's already a stream connected.
   ---------------------------------------------------------------------- *)
PROCEDURE Connect(self : T; file: CharRd.T) RAISES { CharRd.AlreadyConnected };


(* ----------------------------------------------------------------------
   Procedure : Disconnect

   Disconnect the stream from the reader. Return the stream to the caller,
   so that they can use it.
   ---------------------------------------------------------------------- *)
PROCEDURE Disconnect(self : T) : CharRd.T;


(* ----------------------------------------------------------------------
   Procedure : DoCharReader

   Thread application procedure. Loop: wait for signal. if stream attached
   then read from it and set the ready flag (optionally cause interrupt)
   Continue until alerted and then exit.
   ---------------------------------------------------------------------- *)
PROCEDURE DoCharReader(self : T) : REFANY RAISES {};

(* ----------------------------------------------------------------------
   Procedure : ThreadInit

   Initialise the thread closure object. Instantiate the objects and set
   up the pointer to the CPU object. Allocate an interrupt ID to the device
   ---------------------------------------------------------------------- *)
PROCEDURE ThreadInit(self : T; cpu : CPU.T; teletype : BOOLEAN) : T;


END CharReader.




