(* ----------------------------------------------------------------------
   $Id: CharWriter.i3,v 1.4 1995/06/19 14:00:05 sgp13 Exp $

   The object which deals with writing characters. It is more straightforward
   than the reader, as characters are assumed to get written immediately
   ----------------------------------------------------------------------

   $Log: CharWriter.i3,v $
# Revision 1.4  1995/06/19  14:00:05  sgp13
# Optional filter added to remove unwanted characters from output stream
#
# Revision 1.3  1995/03/15  14:45:11  sgp13
# Interrupts now implemented using IntBus.T
#
# Revision 1.2  1995/03/14  12:15:25  sgp13
# Interrupts method altered to counting pending interrupts.
#
# Revision 1.1  1995/03/07  23:06:51  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE CharWriter;

IMPORT Thread, Wr;
IMPORT CPU, IntBus;

EXCEPTION AlreadyConnected;
EXCEPTION NotConnected;

TYPE
  Filter = PROCEDURE(c : CHAR) : CHAR;

  T <: PublicT;
  PublicT = MUTEX OBJECT
    interruptID : IntBus.ID;   (* the interrupt ID for this device *)
    ready : BOOLEAN := FALSE;
    teletype : BOOLEAN;        (* flag whether this is the teletype: if so *)
                               (* clear top bit of character before writing *)
  METHODS
    connect(file : Wr.T) RAISES { AlreadyConnected };
    disconnect() : Wr.T RAISES { Thread.Alerted };
    init(cpu : CPU.T; teletype : BOOLEAN; filter : Filter) : T;
    put(c : CHAR) RAISES { NotConnected, Wr.Failure, Thread.Alerted } ;
  END;


END CharWriter.







