(* ----------------------------------------------------------------------
   $Id: Bits.i3,v 1.7 1995/03/22 12:48:32 sgp13 Exp $

   This module defines the various constants used as bit masks and other
   things referring to parts of a byte.
   ----------------------------------------------------------------------

   $Log: Bits.i3,v $
# Revision 1.7  1995/03/22  12:48:32  sgp13
# BoolToInt and BoolToText added
#
# Revision 1.6  1995/03/14  11:52:43  sgp13
# Bit masks for twiddling individual bits added
#
# Revision 1.5  1995/03/07  21:05:37  sgp13
# lowByte added
#
# Revision 1.4  1995/02/24  22:11:37  sgp13
# CONST clearBitFive added
#
# Revision 1.3  1995/02/21  09:25:54  sgp13
# Implode and Explode procedures added.
#
# Revision 1.2  1995/02/04  13:05:06  sgp13
# Modified to add extra definitions
#
# Revision 1.1  1995/02/01  13:37:48  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE Bits;

CONST
  topBitOfByte =          2_100000000000;
  nextToTopBitOfByte =    2_010000000000;
  nextToBottomBitOfByte = 2_000000000010;
  bottomBitOfByte       = 2_000000000001;

  highHalfOfByte =        2_111111000000;
  lowHalfOfByte  =        2_000000111111;

  lowByte =               2_000011111111;

  twoToTheSixth =              2_1000000;

  setBitZero =            2_100000000000;
  setBitOne =             2_010000000000;
  setBitTwo =             2_001000000000;
  setBitThree =           2_000100000000;
  setBitFour =            2_000010000000;
  setBitFive =            2_000001000000;
  setBitSix =             2_000000100000;
  setBitSeven =           2_000000010000;
  setBitEight =           2_000000001000;
  setBitNine =            2_000000000100;
  setBitTen =             2_000000000010;
  setBitEleven =          2_000000000001;
  clearBitZero =          2_011111111111;
  clearBitOne =           2_101111111111;
  clearBitTwo =           2_110111111111;
  clearBitThree =         2_111011111111;
  clearBitFour =          2_111101111111;
  clearBitFive =          2_111110111111;
  clearBitSix =           2_111111011111;
  clearBitSeven =         2_111111101111;
  clearBitEight =         2_111111110111;
  clearBitNine =          2_111111111011;
  clearBitTen =           2_111111111101;
  clearBitEleven =        2_111111111110;

  (* NB lowBit is the MOST SIGNIFICANT *)
  lowBit = 0;
  highBit = 11;

TYPE
  Range = [lowBit..highBit];
  Array = ARRAY Range OF BOOLEAN;
  IntArray = ARRAY Range OF INTEGER;


(* ----------------------------------------------------------------------
   Procedure : BoolToInt

   Converts a boolean to an integer:

   FALSE -> 0, TRUE -> 1
   ---------------------------------------------------------------------- *)
PROCEDURE BoolToInt(val : BOOLEAN) : INTEGER;

(* ----------------------------------------------------------------------
   Procedure : BoolToText

   Converts a BOOLEAN to "0" or "1" as appropriate
   ---------------------------------------------------------------------- *)
PROCEDURE BoolToText(value : BOOLEAN) : TEXT;

(* ----------------------------------------------------------------------
   Procedure : Explode

   Produce an array of boolean values representing the status of each bit
   of a twelve digit binary number.
   ---------------------------------------------------------------------- *)
PROCEDURE Explode(val : INTEGER) : Array;


(* ----------------------------------------------------------------------
   Procedure : ExplodeToInts

   As for Explode above, but the target array is an array of integers and
   TRUE -> 1, FALSE -> 0

   ---------------------------------------------------------------------- *)
PROCEDURE ExplodeToInts(value : INTEGER) : IntArray;


(* ----------------------------------------------------------------------
   Procedure : Implode

   Generate a 12 bit binary number from an array of 12 boolean values.
   ---------------------------------------------------------------------- *)
PROCEDURE Implode(array : Array) : INTEGER;


END Bits.
