(* ----------------------------------------------------------------------
   $Id: Bits.m3,v 1.7 1995/03/22 12:48:43 sgp13 Exp $

   This implements the procedures associated with the Bits interface.
   These procedures are as documented in that interface.
   ----------------------------------------------------------------------

   $Log: Bits.m3,v $
# Revision 1.7  1995/03/22  12:48:43  sgp13
# BoolToInt and BoolToText added
#
# Revision 1.6  1995/03/14  11:53:06  sgp13
# Update forced
#
# Revision 1.5  1995/03/07  21:06:21  sgp13
# Update forced
#
# Revision 1.4  1995/02/24  22:12:10  sgp13
# Update forced
#
# Revision 1.3  1995/02/21  09:26:59  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE Bits;


(* ----------------------------------------------------------------------
   Procedure : BoolToInt

   Converts a boolean to an integer:

   FALSE -> 0, TRUE -> 1
   ---------------------------------------------------------------------- *)
PROCEDURE BoolToInt(val : BOOLEAN) : INTEGER =
  BEGIN
    IF val = FALSE THEN
      RETURN 0;
    ELSE
      RETURN 1;
    END;
  END BoolToInt;


(* ----------------------------------------------------------------------
   Procedure : BoolToText

   Converts a BOOLEAN to "0" or "1" as appropriate
   ---------------------------------------------------------------------- *)
PROCEDURE BoolToText(value : BOOLEAN) : TEXT =
  BEGIN
    IF value THEN
      RETURN "1";
    ELSE
      RETURN "0";
    END;
  END BoolToText;


(* ----------------------------------------------------------------------
   Procedure : Explode

   Produce an array of boolean values representing the status of each bit
   of a twelve digit binary number.
   ---------------------------------------------------------------------- *)
PROCEDURE Explode(val : INTEGER) : Array =
  VAR
    result : Array;
  BEGIN

    (* work through the bits, least significant first *)
    (* NB on the PDP8 the LSB is bit 11 *)
    FOR i := highBit TO lowBit BY -1 DO
      (* at any stage, the next bit is set iff val is odd *)
      IF (val MOD 2) = 0 THEN
        result[i] := FALSE;
      ELSE
        result[i] := TRUE;
      END;

      val := val DIV 2;
    END;

    RETURN result;
  END Explode;


(* ----------------------------------------------------------------------
   Procedure : ExplodeToInts

   As for Explode above, but the target array is an array of integers and
   TRUE -> 1, FALSE -> 0

   ---------------------------------------------------------------------- *)
PROCEDURE ExplodeToInts(val : INTEGER) : IntArray =
  VAR
    result : IntArray;
  BEGIN

    (* work through the bits, least significant first *)
    (* NB on the PDP8 the LSB is bit 11 *)
    FOR i := highBit TO lowBit BY -1 DO
      (* at any stage, the next bit is set iff val is odd *)
      result[i] := val MOD 2;

      val := val DIV 2;
    END;

    RETURN result;
  END ExplodeToInts;


(* ----------------------------------------------------------------------
   Procedure : Implode

   Generate a 12 bit binary number from an array of 12 boolean values.
   ---------------------------------------------------------------------- *)
PROCEDURE Implode(array : Array) : INTEGER =
  VAR
    val : INTEGER := 0;
  BEGIN
    (* test each bit and feed in at the least significant end of the number *)
    (* and shift upwards *)
    FOR i := lowBit TO highBit DO
      val := val * 2;
      IF array[i] THEN
        INC(val);
      END;
    END;
    RETURN val;

  END Implode;

BEGIN
END Bits.


