(* ----------------------------------------------------------------------
   $Id: BufferRd.m3,v 1.1 1995/06/19 13:49:32 sgp13 Exp $

   This interface specifies the services to be supplied by a reader which
   is connected to a Buffer.T buffer.

   The buffer is expected to be able to be connected or disconnected.

   The object is a descendant of the CharRd type so that it may be specified
   as the 'reader' of a CharReader.
   ----------------------------------------------------------------------

   $Log: BufferRd.m3,v $
# Revision 1.1  1995/06/19  13:49:32  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE BufferRd;

IMPORT Thread;
IMPORT Buffer, CharRd;

REVEAL
  T = PublicT BRANDED OBJECT
    buffer : Buffer.T := NIL;     (* the stream to read from *)
  OVERRIDES
    connect := Connect;
    disconnect := Disconnect;
    get := Get;                (* override the get() method to provide *)
                              (* characters *)
  END;

(* ----------------------------------------------------------------------
   Procedure : Connect

   Connect stream
   ---------------------------------------------------------------------- *)
PROCEDURE Connect(self : T; in : Buffer.T) RAISES { CharRd.AlreadyConnected } =
  BEGIN
    IF self.buffer # NIL THEN
      RAISE CharRd.AlreadyConnected;
    ELSE
      self.buffer := in;
    END;
  END Connect;


(* ----------------------------------------------------------------------
   Procedure : Disconnect

   Disconnect stream
   ---------------------------------------------------------------------- *)
PROCEDURE Disconnect(self : T) : Buffer.T =
  VAR
    temp : Buffer.T;
  BEGIN
    temp := self.buffer;
    self.buffer := NIL;
    RETURN temp;
  END Disconnect;


(* ----------------------------------------------------------------------
   Procedure : Get

   Read character
   ---------------------------------------------------------------------- *)
PROCEDURE Get(self : T) : CHAR RAISES { CharRd.NotConnected, Thread.Alerted } =
  BEGIN
    IF self.buffer = NIL THEN
      RAISE CharRd.NotConnected;
    ELSE
      RETURN self.buffer.get();
    END;
  END Get;

BEGIN
END BufferRd.

