(* ----------------------------------------------------------------------
   $Id: CPU.i3,v 1.4 1995/06/24 21:52:04 sgp13 Exp $

   This interface supplies access to the machine information as seen by the
   CPU.
   It used to be called the CPUdata interface - but is now the CPU interface,
   reflecting that it holds the 'central' data.

   It is used in the closure for both the processor and user interface
   threads.
   ----------------------------------------------------------------------

   $Log: CPU.i3,v $
# Revision 1.4  1995/06/24  21:52:04  sgp13
# Pointers to current fields added for extra speed
#
# Revision 1.3  1995/06/23  07:41:27  sgp13
# Use of Registers.T removed: instead registers are directly part of CPU.T
#
# Revision 1.2  1995/03/15  14:38:29  sgp13
# Interrupt information object changed to IntBus.T
#
# Revision 1.1  1995/02/21  09:29:26  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE CPU;

IMPORT Rd, Thread;
IMPORT Flags, IntBus, (* Registers, *) Memory;

EXCEPTION Quit;

CONST
  OperandSize = 512;

TYPE
  Register_12 = [0..4095];
  Register_3 = [0..7];

  T = OBJECT
    memory : Memory.T;               (* the core memory *)

    (* the registers *)
    AC : Register_12 := 0;
    L : BOOLEAN := FALSE;
    IR : Register_3 := 0;
    MQ : Register_12 := 0;
    PC : Register_12 := 0;
    MB : Register_12 := 0;
    SR : Register_12 := 0;
    SW : BOOLEAN := FALSE;

    (* extended arithmetic - is this ever used ? *)
(*    EAE_SC : Register_5 := 0;
    EAE_IR : Register_4 := 0;
*)
    (* extended memory *)
    (* field numbers *)
    EM_IFnum : Register_3 := 0;
    EM_DFnum : Register_3 := 0;

    (* pointers to memory fields *)
    EM_IFptr : REF Memory.Field;
    EM_DFptr : REF Memory.Field;

    EM_IB : Register_3 := 0;
    EM_SIF : Register_3 := 0;
    EM_SDF : Register_3 := 0;

    run : Flags.Run;                 (* info on run state *)
    interrupt : IntBus.T;            (* info on interrupts *)
    DMA : Flags.DMARequest;          (* info on DMA *)
    quit : Flags.Signal;       (* flag to allow thread to signal to *)
                                     (* control that it is quitting *)
    endOfSingleCycle : Flags.Signal; (* signal to front panel that *)
                                     (* a single step cycle has finished *)
    endOfFreeRunCycle : Flags.Signal;(* signal to front panel that a cycle *)
                                     (* has finished while in free run mode *)
  METHODS
    init(memoryfile : Rd.T) : T RAISES { Memory.Failure, Thread.Alerted } 
                                := ObjectInit;
  END;

  Operand = [0..511];
  OpCode = [0..7];
  

PROCEDURE ObjectInit(self : T; memoryfile : Rd.T := NIL) : T
    RAISES { Memory.Failure, Thread.Alerted };

END CPU.



