(* ----------------------------------------------------------------------
   $Id: CPU.m3,v 1.5 1995/06/24 21:52:23 sgp13 Exp $

   This module supplies access to the machine information as seen by the
   CPU.
   It used to be called the CPUdata module - but is now the CPU module,
   reflecting that it holds the 'central' data.

   It is used in the closure for both the processor and user interface
   threads.

   ----------------------------------------------------------------------

   $Log: CPU.m3,v $
# Revision 1.5  1995/06/24  21:52:23  sgp13
# Pointers to current fields added for extra speed
#
# Revision 1.4  1995/06/23  14:10:08  sgp13
# Bug fix
#
# Revision 1.3  1995/06/23  07:40:38  sgp13
# Use of Registers.T removed: instead registers are directly part of CPU.T
#
# Revision 1.2  1995/03/15  14:38:56  sgp13
# Interrupt data changed to IntBus.T
#
# Revision 1.1  1995/02/21  09:29:41  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE CPU;

IMPORT Rd, Thread;
IMPORT IntBus, Memory, (* Registers, *) Flags;

(* ----------------------------------------------------------------------
   Procedure : ObjectInit

   Initialise CPU object
   ---------------------------------------------------------------------- *)
PROCEDURE ObjectInit(self : T; memoryfile : Rd.T := NIL) : T
      RAISES { Memory.Failure, Thread.Alerted } =
  BEGIN
    self.memory := NEW(Memory.T).init(memoryfile);
(*    self.regsMain := NEW(Registers.Main).init();       *)
(*    self.regsEAE := NEW(Registers.ExtdArith).init();   *)
(*    self.regsExtdMem := NEW(Registers.ExtdMem).init(); *)
    self.run := NEW(Flags.Run).init();
    self.interrupt := NEW(IntBus.T).init();
    self.DMA := NEW(Flags.DMARequest);
    self.quit := NEW(Flags.Signal).init();
    self.endOfSingleCycle := NEW(Flags.Signal).init();
    self.endOfFreeRunCycle := NEW(Flags.Signal).init();

    self.EM_IFptr := self.memory.mem[0];
    self.EM_DFptr := self.memory.mem[0];
    RETURN self;
  END ObjectInit;

BEGIN
END CPU.






