(* ----------------------------------------------------------------------
   $Id: Diag.i3,v 1.12 1995/06/21 13:56:31 sgp13 Exp $

   Diagnostics information handler for the PDP8/e emulator.

   This module contains the diagnostics routines. It is runtime
   configurable to write diagnostics information to a file and / or the
   standard output. Also configured at runtime is the classes of diagnostics
   messages which are produced. All the configuration is via the
   initialisation file specified on the command line and handled by the
   Init module.


   This module reads from two sections of the initialisation file :

   [DiagSetup]
   File=<filename>          - file to write to
   Stdout=TRUE              - whether to write to stdout

   [Diag]
   <message class>=TRUE | FALSE

   The classes of diagnostics messages are given below, along with their
   corresponding strings in the initialisation file. For example, to enable
   memory module diagnostics, include a line reading:

   Memory=TRUE

   in the [Diag] section of the initialisation file.
   NB It is not advisable to disable error messages (which are enabled by
      default).


   To produce a diagnostics message, call the Message procedure. If messages
   of this class are enabled then the message will be produced.


   Before exiting the program Close should be run to allow the module to
   close the diagnostics file gracefully.

   ----------------------------------------------------------------------

   $Log: Diag.i3,v $
# Revision 1.12  1995/06/21  13:56:31  sgp13
# Update forced
#
# Revision 1.11  1995/03/22  12:49:13  sgp13
# GraphicalUI message class added
#
# Revision 1.10  1995/03/07  21:07:25  sgp13
# CharDevice class added
#
# Revision 1.9  1995/02/19  15:12:08  sgp13
# CPU type changed to Processor - commensurate with changing names of modules.
#
# Revision 1.8  1995/02/10  12:08:21  sgp13
# New classes ( Bus, Device, Teletype, Tape, ExtdMem, Disc, Oscilloscope )
#
# Revision 1.7  1995/02/01  13:24:25  sgp13
# New classes ( InterpretedOPR and CompiledOPR ) added
#
# Revision 1.6  1995/01/19  09:19:42  sgp13
# All output now immediately flushed to file.
#
# Revision 1.5  1995/01/12  15:10:07  sgp13
# Message classes KeyboardUI, GeneralUI and Control added
#
# Revision 1.4  1994/12/09  18:16:08  sgp13
# Added Test message class
#
# Revision 1.3  1994/12/09  11:46:42  sgp13
# Changed to ignore Thread.Alerted exception
#
# Revision 1.2  1994/12/09  11:20:33  sgp13
# New message classes added : DMA, Registers, Flags, Interrupt
#
# Revision 1.1  1994/12/06  12:05:17  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)

INTERFACE Diag;


TYPE
  (* the possible classes of diagnostics message *)
  T = { Memory, Processor, Instruction, Error, DMA, Registers, Flags,
        Interrupt, Test, KeyboardUI, GeneralUI, Control, InterpOPR, PrecompOPR,
        Bus, Device, CharDevice, Teletype, Tape, ExtdMem, Disc, Oscilloscope,
        GraphicalUI };

VAR
  (* the string to put in the init file for each message class *)
  varname := ARRAY T OF TEXT { "Memory", "Processor", "Instruction", "Error",
        "DMA", "Registers", "Flags", "Interrupt", "Test", "KeyboardUI",
        "GeneralUI", "Control", "InterpretedOPR", "CompiledOPR",
        "Bus", "Device", "CharDevice", "Teletype", "Tape", "ExtdMem", "Disc",
        "Oscilloscope", "GraphicalUI" };


(* ----------------------------------------------------------------------
   Procedure : Message

   Produces a message of the class given, containing the text given, if
   messages of this class are enabled. Ignores any Thread.Alerted exceptions
   as alerts will be picked up by Thread.TestAlert
   ---------------------------------------------------------------------- *)
PROCEDURE Message(class : T; message : TEXT);


(* ----------------------------------------------------------------------
   Procedure : Close

   Close the diagnostics output file and flush standard output if
   appropriate. Ignores any Thread.Alerted exceptions
   as alerts will be picked up by Thread.TestAlert
   ---------------------------------------------------------------------- *)
PROCEDURE Close();

END Diag.




