(* ----------------------------------------------------------------------
   $Id: Memory.i3,v 1.12 1995/06/24 21:53:10 sgp13 Exp $

   Interface defining the object which is the core memory of the emulator.
   This object is created by the control thread and a reference passed to
   all threads which have the need to access the core memory.

   The possible actions on memory are:

   init - either from given file or as a blank memory
   dump - to given file
   read - read a single byte : specified by field and address
   write - write a single byte : specified by field and address
   
   The responsibility for auto-increment and for indirection is on the
   code calling this module - the Memory module implements neither.

   Efficient DMA access will need the ability to access a block of memory.
   There is a mutex on memory which is used to ensure that only one request
   for memory access is satisfied at any one time.

   ----------------------------------------------------------------------

   $Log: Memory.i3,v $
# Revision 1.12  1995/06/24  21:53:10  sgp13
# Memory split into fields so that current field pointers may be used to speed
# access
#
# Revision 1.11  1995/06/22  13:59:15  sgp13
# Memory made direct access (rather than via object methods)
#
# Revision 1.10  1995/06/21  16:06:55  sgp13
# *** empty log message ***
#
# Revision 1.9  1995/06/21  13:57:21  sgp13
# Unncessary RAISES { Thread.Alerted } clauses removed.
#
# Revision 1.8  1995/06/20  13:23:52  sgp13
# Memory is no longer a MUTEX.
# Also make the memory block a direct member of the object, rather than a
# dynamically allocated reference - to save time.
# This means Pickle must pickle the object rather than the memory reference, but
# this should be OK and is not time critical.
#
# Revision 1.7  1995/02/01  13:26:45  sgp13
# new constant: halfByteSize = byteSize DIV 2
#
# Revision 1.6  1995/01/18  13:18:58  sgp13
# Diagnostics values now padded to four figures where appropriate
# (dummy check-in for interface file)
#
# Revision 1.5  1995/01/18  13:14:23  sgp13
# dump now flushes output stream
#
# Revision 1.4  1995/01/11  14:21:42  sgp13
# Added inc() method to increment location and then read
#
# Revision 1.3  1995/01/11  10:22:08  sgp13
# Changed for new memory protocol, where this module is ONLY responsible for
# reads and writes of specified addresses.
#
# Revision 1.2  1994/12/12  16:41:00  sgp13
# Bug in Memory.T.init() fixed - now returns itself
#
# Revision 1.1  1994/12/07  12:38:05  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
INTERFACE Memory;

IMPORT Thread, Rd, Wr;

EXCEPTION Failure;

CONST
  fieldSize = 4096;
  maxFields = 8;
  byteSize = 4096;
  halfByteSize = byteSize DIV 2;

TYPE
  Address = [0..fieldSize - 1];
  Value = [0..byteSize - 1];
  FieldNumber = [0..maxFields - 1];
  Field = ARRAY Address OF Value;


(* these types and constants are not currently used *)
(* they will be necessary if memory is partitioned into pages *)
(*
  CONST
    pageSize = 128;
    pagesPerField = fieldSize DIV pageSize;
  TYPE
    PageOffset = [0..pageSize - 1];
    PageType = { Current, Zero };
    PageNumber = [0..pagesPerField - 1];
    FieldType = { Instruction, Data };
*)

TYPE
  T <: Public;               (* the memory object - with methods as below *)
  Public = OBJECT
    mem : ARRAY FieldNumber OF REF Field;
  METHODS
    init(source : Rd.T := NIL) : T RAISES { Failure, Thread.Alerted };
    dump(sink : Wr.T) RAISES { Failure, Thread.Alerted };
(*    inc(address : Address; field : FieldNumber) : Value;
    read(address : Address; field : FieldNumber) : Value;
    write(address : Address; field : FieldNumber; value : Value); *)
  END;

END Memory.











