(* ----------------------------------------------------------------------
   $Id: StreamRd.m3,v 1.1 1995/06/19 13:50:32 sgp13 Exp $

   This interface specifies the services to be supplied by a reader which
   is connected to a non-intermittent Unix file stream (ie reading from
   a normal file)

   The stream is expeceted to be able to be connected or disconnected.

   The object is a descendant of the CharRd type so that it may be specified
   as the 'reader' of a CharReader.
   ----------------------------------------------------------------------

   $Log: StreamRd.m3,v $
# Revision 1.1  1995/06/19  13:50:32  sgp13
# Initial revision
#

   ---------------------------------------------------------------------- *)
MODULE StreamRd;

IMPORT Rd;
IMPORT CharRd;

REVEAL
  T = PublicT BRANDED OBJECT
    stream : Rd.T := NIL;     (* the stream to read from *)
  OVERRIDES
    connect := Connect;
    disconnect := Disconnect;
    get := Get;               (* override the get() method to provide *)
                              (* characters *)
  END;

(* ----------------------------------------------------------------------
   Procedure : Connect

   Connect stream
   ---------------------------------------------------------------------- *)
PROCEDURE Connect(self : T; in : Rd.T) RAISES { CharRd.AlreadyConnected } =
  BEGIN
    IF self.stream # NIL THEN
      RAISE CharRd.AlreadyConnected;
    ELSE
      self.stream := in;
    END;
  END Connect;


(* ----------------------------------------------------------------------
   Procedure : Disconnect

   Disconnect stream
   ---------------------------------------------------------------------- *)
PROCEDURE Disconnect(self : T) : Rd.T =
  VAR
    temp : Rd.T;
  BEGIN
    temp := self.stream;
    self.stream := NIL;
    RETURN temp;
  END Disconnect;


(* ----------------------------------------------------------------------
   Procedure : Get

   Read character
   ---------------------------------------------------------------------- *)
PROCEDURE Get(self : T) : CHAR RAISES { CharRd.NotConnected } =
  BEGIN
    IF self.stream = NIL THEN
      RAISE CharRd.NotConnected;
    ELSE
      RETURN Rd.GetChar(self.stream);
    END;
  END Get;

BEGIN
END StreamRd.
